package com.qmth.distributed.print.business.activiti;

import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.entity.TFFlowApprove;
import com.qmth.distributed.print.business.enums.MessageEnum;
import com.qmth.distributed.print.business.service.BasicMessageService;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.distributed.print.business.service.TFFlowApproveService;
import com.qmth.teachcloud.common.bean.params.ApproveUserResult;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FlowGdykdxApproveSetupEnum;
import com.qmth.teachcloud.common.enums.FlowStatusEnum;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.JacksonUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.activiti.engine.delegate.DelegateTask;
import org.activiti.engine.delegate.TaskListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description: 教学院长审批
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/7/29
 */
public class PresidentApproveListener implements TaskListener {
    private final static Logger log = LoggerFactory.getLogger(PresidentApproveListener.class);

    /**
     * 查找同级和以上人员
     *
     * @param delegateTask
     */
    @Override
    public void notify(DelegateTask delegateTask) {
        SysUserService sysUserService = SpringContextHolder.getBean(SysUserService.class);
        ExamTaskService examTaskService = SpringContextHolder.getBean(ExamTaskService.class);
        BasicMessageService basicMessageService = SpringContextHolder.getBean(BasicMessageService.class);
        TFFlowApproveService tfFlowApproveService = SpringContextHolder.getBean(TFFlowApproveService.class);
        List<Long> approveUserIds = (List<Long>) delegateTask.getVariable(SystemConstant.APPROVE_PRESIDENT_USER_IDS);
        String flowId = delegateTask.getProcessInstanceId();
        ExamTask examTask = examTaskService.findByFlowId(Long.parseLong(flowId));
        List<ApproveUserResult> sysUserList = null;
        if (Objects.nonNull(approveUserIds) && approveUserIds.size() > 0) {
            List<SysUser> sysUsers = sysUserService.listByIds(approveUserIds);
            sysUserList = new Gson().fromJson(JacksonUtil.parseJson(sysUsers), new TypeToken<List<ApproveUserResult>>() {
            }.getType());
            delegateTask.addCandidateUsers(approveUserIds.stream().map(x -> String.valueOf(x)).collect(Collectors.toList()));
        } else {
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            if (Objects.isNull(sysUser.getOrgId())) {
                throw ExceptionResultEnum.ERROR.exception("审批人机构为空");
            }
            sysUserList = sysUserService.findByPresidentApprove(Arrays.asList(sysUser.getOrgId()), null);
            if (Objects.nonNull(sysUserList) && sysUserList.size() > 0) {
                List<String> ids = sysUserList.stream().map(s -> String.valueOf(s.getId())).collect(Collectors.toList());
                //添加或签的人员，一人通过即可进入下一环节
                delegateTask.addCandidateUsers(ids);
            }
        }
        // 发送审核短信
        TFFlowApprove tfFlowApprove = tfFlowApproveService.findByFlowId(Long.valueOf(flowId));
        if (tfFlowApprove.getStatus().equals(FlowStatusEnum.AUDITING)
                && tfFlowApprove.getSetup().equals(FlowGdykdxApproveSetupEnum.SECOND_APPROVE.getSetup())) {
            basicMessageService.sendNoticeTaskAuditFlow(examTask, sysUserList, MessageEnum.NOTICE_OF_AUDIT_CREATED);
        }
    }
}