package com.qmth.distributed.print.business.templete.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.distributed.print.business.templete.service.PushLogicService;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.bean.params.PushBeforeRoleParam;
import com.qmth.teachcloud.common.bean.params.UserPushParam;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.userPush.BeforeJudgeEnum;
import com.qmth.teachcloud.common.enums.userPush.SyncStatusEnum;
import com.qmth.teachcloud.common.service.SysUserRoleService;
import com.qmth.teachcloud.common.service.SysUserService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description: 同步推送处理逻辑接口实现类
 * @Author: CaoZixuan
 * @Date: 2021-11-01
 */
@Service
public class PushLogicServiceImpl implements PushLogicService {
    @Resource
    private SysUserService sysUserService;
    @Resource
    private SysUserRoleService sysUserRoleService;

    @Override
    public Map<String, Object> executeUserPushLogic(Map<String, Object> map) throws IllegalAccessException {
        int correct = 0;
        int count = 0;
        SysUser sysUser = (SysUser) map.get(SystemConstant.SYS_USER);
        Long schoolId = sysUser.getSchoolId();
        // 需要同步的用户
        List<SysUser> sysUserList = sysUserService.list(new QueryWrapper<SysUser>().lambda()
                .eq(SysUser::getSchoolId,schoolId)
                .eq(SysUser::getSyncStatus, SyncStatusEnum.NEED_PUSH_AGAIN));
        count = sysUserList.size();
        for (SysUser user : sysUserList) {
            Set<Long> roleIdSet = sysUserRoleService.listRoleByUserId(user.getId()).stream().map(BaseEntity::getId).collect(Collectors.toSet());
            PushBeforeRoleParam pushBeforeRoleParam = new PushBeforeRoleParam(null, BeforeJudgeEnum.FORBIDDEN_NECESSARY);
            List<UserPushParam> userPushParamList = sysUserService.analyzeUserPushSpecialPrivilege(user.getId(),user.getSchoolId(),user.getCode(), user.getRealName(),user.getPassword(),roleIdSet,pushBeforeRoleParam,user.getEnable());
            boolean syncResult = sysUserService.userPushService(userPushParamList,sysUser);
            if (syncResult){
                correct ++;
            }
            UpdateWrapper<SysUser> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(SysUser::getSyncStatus,null).eq(SysUser::getId,user.getId());
            sysUserService.update(updateWrapper);
        }
        map.put("correct",correct);
        map.put("count",count);
        return map;
    }
}
